#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
#
# Author:   lambdalisue (lambdalisue@hashnote.net)
# URL:      http://hashnote.net/
# License:  MIT license
# Created:  2013-01-23
#
from numpy import sqrt
MICRO = u"\u00b5"
SIGMA = u"\u03c3"
PLMIN = u"\u00b1"


def create_formula(X, model, index=0):
    w = model.weights_[index]
    m = model.means_[index][0]
    c = model.covars_[index][0]  # covariance = variance in 1-dimension
    d = sqrt(c)                  # SD = sqrt(variance)
    e = d / sqrt(len(X))         # SE = SD / sqrt(N)
    f = "%.2f%%: %s=%.2e%s%.2e, %s=%.2e"
    f = f % (w * 100, MICRO, m, PLMIN, e, SIGMA, d)
    return f



#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
#
# Author:   lambdalisue (lambdalisue@hashnote.net)
# URL:      http://hashnote.net/
# License:  MIT license
# Created:  2013-01-23
#
from utils import create_formula


def fit(data, model, criterions, opts):
    print "# N = %d, Bins = %d, Base = %.2e" % (
            len(data), opts.bins, opts.base)
    print "n_components_try = %d, n_components_best = %d" % (
            len(criterions[0]),
            len(model.weights_),
        )
    
    for i in range(len(model.weights_)):
        formula = create_formula(data, model, i)
        print formula.encode('utf-8')
    return data, model, criterions, opts


def histogram(data, model, criterions, opts):
    from numpy import histogram
    print "# N = %d, Bins = %d, Base = %.2e" % (
            len(data), opts.bins, opts.base)
    hist, bin_edges = histogram(data, bins=opts.bins)
    for index in range(len(hist)):
        print bin_edges[index], hist[index]
    return data, model, criterions, opts



#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
#
# Author:   lambdalisue (lambdalisue@hashnote.net)
# URL:      http://hashnote.net/
# License:  MIT license
# Created:  2013-01-23
#
from numpy import zeros, argmin
from sklearn.mixture import GMM


def fit(data, n_components, covariance_type='diag', min_covar=1e-3):
    # create n_components of GMM
    models = zeros(n_components, dtype=object)
    for i in range(n_components):
        kwargs = dict(
            covariance_type=covariance_type,
            min_covar=min_covar)
        models[i] = GMM(i+1, **kwargs).fit(data)

    # calculate AIC, BIC
    AIC = [m.aic(data) for m in models]
    BIC = [m.bic(data) for m in models]
    # find best match by AIC
    best = models[argmin(AIC)]
    return best, (AIC, BIC)



#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
#
# Author:   lambdalisue (lambdalisue@hashnote.net)
# URL:      http://hashnote.net/
# License:  MIT license
# Created:  2013-01-23
#
from decimal import Decimal
from numpy import vectorize

import prefix

ONE = Decimal('1')

def flatten(l):
    if isinstance(l, (list, tuple)):
        return reduce(lambda a,b: a + flatten(b), l, [])
    else:
        return [l]


def modulate_base(value, base=ONE):
    if not isinstance(base, Decimal):
        base = Decimal(base)
    return Decimal(value) / base
modulate_base = vectorize(modulate_base)


def to_float(value):
    return float(value)
to_float = vectorize(to_float)



#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
#
# Author:   lambdalisue (lambdalisue@hashnote.net)
# URL:      http://hashnote.net/
# License:  MIT license
# Created:  2013-01-23
#
import random
import numpy as np
from decimal import Decimal
from sklearn.mixture import GMM


def demo():
    np.random.seed(1)
    gmm = GMM(3, n_iter=1)
    gmm.means_ = np.array([[-1], [0], [3]])
    gmm.covars_ = np.array([[1.5], [1], [0.5]]) ** 2
    gmm.weights_ = np.array([0.3, 0.5, 0.2])
    return gmm.sample(1000)



#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
#
# Author:   lambdalisue (lambdalisue@hashnote.net)
# URL:      http://hashnote.net/
# License:  MIT license
# Created:  2013-01-23
#
from decimal import Decimal

ONE = Decimal('1')
NEGATIVES = (
    ('y',    Decimal('1e-24')),
    ('z',    Decimal('1e-21')),
    ('a',    Decimal('1e-18')),
    ('f',    Decimal('1e-15')),
    ('p',    Decimal('1e-12')),
    ('n',    Decimal('1e-09')),
    ('u',    Decimal('1e-06')),
    ('m',    Decimal('1e-03')),
    ('c',    Decimal('1e-02')),
    ('d',    Decimal('1e-01')),
)
POSITIVES = (
    ('Y',    Decimal('1e+24')),
    ('Z',    Decimal('1e+21')),
    ('E',    Decimal('1e+18')),
    ('P',    Decimal('1e+15')),
    ('T',    Decimal('1e+12')),
    ('G',    Decimal('1e+09')),
    ('M',    Decimal('1e+06')),
    ('k',    Decimal('1e+03')),
    ('h',    Decimal('1e+02')),
    ('da',   Decimal('1e+01')),
)

def get_prefix(value):

    if not isinstance(value, Decimal):
        value = Decimal(value)
    abs_value = abs(value)
    if abs_value > ONE:
        for (prefix, threshold) in POSITIVES:
            if threshold <= abs_value:
                return value / threshold, prefix
    elif abs_value < ONE:
        previous_prefix = ''
        previous_threshold = Decimal(0)
        for (prefix, threshold) in NEGATIVES:
            if threshold > abs_value:
                return value / previous_threshold, previous_prefix
            previous_prefix = prefix
            previous_threshold = threshold
    return value, ''


if __name__ == '__main__':
    assert get_prefix(Decimal('1000')) == (ONE, 'k')
    assert get_prefix(Decimal('0.001')) == (ONE, 'm')
    assert get_prefix(Decimal('-4e+25')) == (Decimal('-40'), 'Y')
    assert get_prefix(Decimal('-2e-23')) == (Decimal('-20'), 'y')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

version = "0.1.4"

def read(filename):
    import os.path
    return open(os.path.join(os.path.dirname(__file__), filename)).read()
setup(
    name="histogramy",
    version=version,
    description = "A small program to analysis 1 dimensional data",
    long_description=read('README.md'),
    classifiers = [
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ],
    keywords = "1-dimensional, analysis, histogram",
    author = "Alisue",
    author_email = "lambdalisue@hashnote.net",
    url=r"https://github.com/lambdalisue/histogramy",
    download_url = r"https://github.com/lambdalisue/histogramy/tarball/master",
    license = 'MIT',
    packages = find_packages('src'),
    package_dir = {'': 'src'},
    include_package_data = True,
    zip_safe = True,
    install_requires=[
        'setuptools',
        'numpy', 'matplotlib',
        'scikit-learn',
    ],
    entry_points={
        'console_scripts': [
            'histogramy = histogramy.main:main',
        ],
    },
)



#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
#
# Author:   lambdalisue (lambdalisue@hashnote.net)
# URL:      http://hashnote.net/
# License:  MIT license
# Created:  2013-01-23
#
import sys
import argparse
from decimal import Decimal
import prefix
import describer


ONE = Decimal(1)
LEGEND_LOCATIONS = (
    'upper right',
    'upper left',
    'lower left',
    'lower right',
    'right',
    'center left',
    'center right',
    'lower center',
    'upper center',
    'center',
)


def plot(data, model, criterions, opts):
    import plotter
    return plotter.plot(data, model, criterions, opts)


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument('-b', '--bins', default=10, type=int, metavar='BINS',
        help='It defines the number of equal-width bins.')
    parser.add_argument('-c', '--column', default=0, type=int, metavar='N',
        help='A number of column in data file used for analysis')
    parser.add_argument('-C', '--classifiers', default=10, type=int,
        metavar='N',
        help='The maximum number classifiers to simulate the fitting')
    parser.add_argument('--base', default=Decimal('1'), metavar='BASE',
        help='Base value to modulate the data')
    parser.add_argument('--auto-base', default=False, action='store_false',
        help='Automatically find the base value to modulate the data')
    parser.add_argument('--min-threshold', metavar='MIN', type=Decimal,
        help='Minimum threshold. Value smaller than this will be ignored')
    parser.add_argument('--max-threshold', metavar='MAX', type=Decimal,
        help='Maximum threshold. Value grater than this will be ignored')
    parser.add_argument('--covariance-type', default='diag', metavar='TYPE',
        help='Type of covariance. Default is "diag"')
    parser.add_argument('--min-covar', default=1e-3, type=float,
        metavar='MIN_COVAR',
        help='Minimum value of covariance')
    parser.add_argument('--delimiter', default=',', metavar='DELIMITER',
        help='Delimiter used to parse the data file')
    parser.add_argument('--encoding', default='utf-8', metavar='ENCODING',
        help='Encoding used to open the data file')
    parser.add_argument('--demo', default=False, action='store_true',
        help='Use demo data to analysis')
    parser.add_argument('filenames', default=[], nargs='*')

    # subparsers
    sp = parser.add_subparsers()

    # create the subparser for the 'histogram' command
    parser_histogram = sp.add_parser('histogram',
        help='Show histogram data')
    parser_histogram.set_defaults(func=describer.histogram)
    # create the subparser for the 'fit' command
    parser_fit = sp.add_parser('fit',
        help='Show fitting data')
    parser_fit.set_defaults(func=describer.fit)
    # create the subparser for the 'plot' command
    parser_plot = sp.add_parser('plot',
        help='Create graph by matplotlib')
    parser_plot.add_argument('-o', '--output', metavar='FILE',
        help='Export the graph as a PNG image to the FILE')
    parser_plot.add_argument('--graph-width', default=8, type=int,
        help='The width (inch) of the graph')
    parser_plot.add_argument('--graph-height', default=6, type=int,
        help='The height (inch) of the graph')
    parser_plot.add_argument('--graph-dpi', default=100, type=int,
        help='The DPI of the graph')
    parser_plot.add_argument('--graph-facecolor', default='white',
        help='The face color of the graph')
    parser_plot.add_argument('--graph-edgecolor', default='#3d3d3d',
        help='The edge color of the graph')
    parser_plot.add_argument('--graph-textcolor', default='#3d3d3d',
        help='The text color of the graph')
    # histogram
    parser_plot.add_argument('--histogram-type',
        default='stepfilled',
        choices=['bar', 'barstacked', 'step', 'stepfilled'],
        help='The type of histogram to draw')
    parser_plot.add_argument('--histogram-color',
        default='#6a8cc7', metavar='COLOR',
        help='The color of histogram to draw')
    parser_plot.add_argument('--histogram-edgecolor',
        default='#3261ab', metavar='COLOR',
        help='The edge color of histogram to draw')
    parser_plot.add_argument('--histogram-linewidth',
        default=1, type=int,
        help='The edge width of histogram to draw')
    parser_plot.add_argument('--histogram-alpha', type=float,
        default=1.0, metavar='ALPHA',
        help='The transparency of histogram to draw')
    parser_plot.add_argument('--histogram-xlabel',
        default='X', metavar='LABEL',
        help='The label of histogram X axis')
    parser_plot.add_argument('--histogram-ylabel',
        default='Frequency', metavar='LABEL',
        help='The label of histogram Y axis')
    parser_plot.add_argument('--histogram-xmin', type=float,
        metavar='MIN',
        help='The minimum value of histogram X axis to draw')
    parser_plot.add_argument('--histogram-xmax', type=float,
        metavar='MAX',
        help='The maximum value of histogram X axis to draw')
    parser_plot.add_argument('--histogram-ymin', type=float,
        metavar='MIN',
        help='The minimum value of histogram Y axis to draw')
    parser_plot.add_argument('--histogram-ymax', type=float,
        metavar='MAX',
        help='The maximum value of histogram Y axis to draw')
    parser_plot.add_argument('--histogram-legend',
        choices=LEGEND_LOCATIONS, nargs='?', const='upper left',
        help='The location of the histogram legend')
    parser_plot.add_argument('--histogram-legend-size', type=int, default=8,
        help='The size of the histogram legend')
    # fitting
    parser_plot.add_argument('--no-fitting', dest='draw_fitting',
        default=True, action='store_false',
        help='Do not draw fitting curves')
    parser_plot.add_argument('--fitting-style',
        default='-', metavar='STYLE',
        help='The style of fitting curve to draw')
    parser_plot.add_argument('--fitting-color',
        default='#c7243a', metavar='COLOR',
        help='The color of fitting curve to draw')
    parser_plot.add_argument('--fitting-alpha', type=float,
        default=0.8, metavar='ALPHA',
        help='The transparency of fitting curve to draw')
    parser_plot.add_argument('--fitting-linewidth',
        default=1, type=int,
        help='The line width of fitting curve to draw')
    parser_plot.add_argument('--fitting-style-individual',
        default='--k', metavar='STYLE',
        help='The style of individual fitting curve to draw')
    parser_plot.add_argument('--fitting-color-individual',
        default='black', metavar='COLOR',
        help='The color of individual fitting curve to draw')
    parser_plot.add_argument('--fitting-alpha-individual', type=float,
        default=0.7, metavar='ALPHA',
        help='The transparency of individual fitting curve to draw')
    parser_plot.add_argument('--fitting-linewidth-individual',
        default=2, type=int,
        help='The line width of individual fitting curve to draw')
    parser_plot.add_argument('--fitting-ylabel',
        default='Probability', metavar='LABEL',
        help='The label of fitting curve Y axis')
    parser_plot.add_argument('--fitting-label',
        default='Fitting curve', metavar='LABEL',
        help='The legend label of fitting curve')
    parser_plot.add_argument('--fitting-ymin', type=float,
        metavar='MIN',
        help='The minimum value of fitting curve Y axis to draw')
    parser_plot.add_argument('--fitting-ymax', type=float,
        metavar='MAX',
        help='The maximum value of fitting curve Y axis to draw')
    parser_plot.add_argument('--fitting-legend',
        choices=LEGEND_LOCATIONS, nargs='?', const='upper right',
        help='The location of the fitting curve legend')
    parser_plot.add_argument('--fitting-legend-size', type=int, default=8,
        help='The size of the fitting curve legend')
    # criterions
    parser_plot.add_argument('--criterions', dest='draw_criterions',
        default=False, action='store_true',
        help='Plot AIC and BIC values of each # of components in subfigure')
    parser_plot.add_argument('--criterions-xlabel',
        default='# of components', metavar='LABEL',
        help='The label of criterions graph X axis')
    parser_plot.add_argument('--criterions-style-aic',
        default='-k', metavar='STYLE',
        help='The style of AIC to draw')
    parser_plot.add_argument('--criterions-alpha-aic', type=float,
        default=1.0, metavar='ALPHA',
        help='The transparency of AIC to draw')
    parser_plot.add_argument('--criterions-style-bic',
        default='--k', metavar='STYLE',
        help='The style of BIC to draw')
    parser_plot.add_argument('--criterions-alpha-bic', type=float,
        default=1.0, metavar='ALPHA',
        help='The transparency of BIC to draw')
    parser_plot.set_defaults(func=plot)

    opts = parser.parse_args(args)

    # delimiter can be specified as SPACE, TAB, \s, \t
    delimiter = opts.delimiter.lower()
    if delimiter in ('space', '\\s'):
        opts.delimiter = ' '
    elif delimiter in ('tab', '\\t'):
        opts.delimiter = '\t'
    # convert SI Prefix to Decimal
    if isinstance(opts.base, basestring):
        # SI Prefix -> Decimal
        ps = list(prefix.NEGATIVES) + list(prefix.POSITIVES)
        ps = dict(ps)
        opts.base = ps.get(opts.base, None) or Decimal(opts.base)
    return opts
